# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

import board
import terminalio
import busio
import displayio
from vectorio import Rectangle, Circle
from adafruit_display_text import label
from adafruit_st7789 import ST7789
import digitalio
import analogio
import time
import random

# object for the mic info
mic = analogio.AnalogIn(board.GP26_A0)

# Release any resources currently in use for the displays
displayio.release_displays()
# SPI pins for display st7789
tft_cs = board.GP17
tft_dc = board.GP16
# setup spi bus
spi = busio.SPI(board.GP10, board.GP11)
# ============ setup display ================== #
display_bus = displayio.FourWire(spi, command=tft_dc,
                                 chip_select=tft_cs, reset=board.GP18)
display = ST7789(display_bus,
                 width=240,
                 height=240,
                 rowstart=80,
                 auto_refresh=True)

# ============== Make the display context =========== #
screen = displayio.Group()
palette = displayio.Palette(1)

circle = Circle(pixel_shader=palette, radius=1, x=120, y=120)
font = terminalio.FONT
ampInfo = label.Label(font, text="", color=0x555555)
ampInfo.x, ampInfo.y = 10, 10
maxInfo = label.Label(font, text="", color=0xFF0000)
screen.append(circle)
screen.append(ampInfo)
screen.append(maxInfo)
display.show(screen)

# ============ button set up ========== #

buttonOne = digitalio.DigitalInOut(board.GP13)
buttonOne.switch_to_input(pull=digitalio.Pull.DOWN)
buttonTwo = digitalio.DigitalInOut(board.GP14)
buttonTwo.switch_to_input(pull=digitalio.Pull.DOWN)

# ============== functions for conversion in True loop ===============


def get_voltage(raw):
    return (raw * 3.3) / 65536.0


def mapFromTo(v, oldMin, oldMax, newMin, newMax):
   newV = (v-oldMin)/(oldMax-oldMin)*(newMax-newMin)+newMin
   return newV


def average(lst):
    return sum(lst) / len(lst)


def rgb2hex(r, g, b):
    return eval("0x{:02x}{:02x}{:02x}".format(r, g, b))


def hex2rgb(hexcode):
    return tuple(map(ord, hexcode[1:].decode('hex')))


palette[0] = rgb2hex(255, 0, 0)

# ============== set up some variables ===================== #

# holder of "old mic value" (so we can compare each loop)
oldAmp = mic.value
minRad = 1
# initial "minimal radius" for converted value of amp (mic.value)
convertedRad = minRad
# a range of "change" that allows an input to be ignored if the change isn't massive
buffer = 1000
# my mic's lowest registered sound is 23500 so I've rounded it to 24000
lowest = 25000
# highest val is 36500 so I took it down to avoid the largest cliping input
highest = 35000
maxNoise = 0

ampInfo.text = "radius = {}".format(convertedRad)
while True:
    #get the mic value
    amp = mic.value
    # check to see if the current average amp is inside the acceptable previous range (so we won't change the display)
    if lowest <= amp <= highest:
        if (oldAmp - buffer) <= amp <= (oldAmp + buffer):
            pass
        else:
            convertedRad = int(mapFromTo(amp, lowest, highest, 1, 120))
            if convertedRad > maxNoise:
                maxNoise = convertedRad
                maxInfo.text = "max = {}".format(maxNoise)
            #print("amp = {} rad = {}".format(avAmp, convertedRad))
            palette[0] = rgb2hex(convertedRad*2 + 10, 255-convertedRad*2, 0)
            circle.radius = convertedRad
            ampInfo.text = "radius = {}".format(convertedRad)
            oldAmp = amp

    if buttonTwo.value:
        maxInfo.x, maxInfo.y = 10, 230
    else:
        maxInfo.x, maxInfo.y = -1000, -1000

    if buttonOne.value:
        maxNoise = 0
        maxInfo.text = "reset"
        maxInfo.x, maxInfo.y = 10, 230
        time.sleep(0.5)
        maxInfo.x, maxInfo.y = -1000, -1000
    #print("amp = {} & oldAmp = {} low = {} hi = {} av = {}".format(amp, oldAmp, lowest, highest, average(micArray)))
